Scriptname pppDrunkenDragonInnQuestScript extends Quest  

; Properties
Faction Property DismissedFollowerFaction Auto
ObjectReference Property pppDrunkenDragonSafeRefA  Auto  
ObjectReference Property pppDrunkenDragonChestRefA  Auto  
Quest Property pppDrunkenDragonInnQuest Auto
Quest Property DialogueFollower Auto
ReferenceAlias Property pppInnShopkeeperRefA Auto
MiscObject Property Gold  Auto  
GlobalVariable Property GameHour  Auto  

; Functions
; Sets the update interval in game time for the next update
Function SetUpdate()
	float fRand = Utility.RandomFloat(1.0, 4.0)
	RegisterForSingleUpdateGameTime(fRand)
endFunction

; Selects a random item in the items chest, gets its value, 
; removes the item from the chest and adds the value to the
; store's safe.
; Let's consider a "sale" to be one OnUpdateGameTime
; The While() loop stops the sale from continuing once the
; sale's value is greater than 100 gold. Each sale can have
; as many transactions as it takes to reach 100 gold.
; This is intended to help meter the speed at which items
; sell so that it can't be abused too much.
Event OnUpdateGameTime()
	if (GameHour.GetValue() > 8 && GameHour.GetValue() < 20)
		int iTotalValue = 0
		int iTotalItems = 0
		int iItemValue = 0
		int iCount = 1
		while (iTotalValue < 100)
			int iItemIndex = enumItems()
			if (iItemIndex < 0)
				iTotalValue = 101
			else
				form oItem = pppDrunkenDragonChestRefA.GetNthForm(iItemIndex)
				; If the selected item is an ammo type, sell up to 20 at a time, otherwise sell 1
				if (oItem.getType() == 42) ;kAmmo
					if (pppDrunkenDragonChestRefA.GetItemCount(oItem) > 20)
						iCount = 20
					else
						;iCount = pppDrunkenDragonChestRefA.GetItemCount(oItem)
						iCount = 1
					endIf
				endIf
				; Call GetPrice() to calculate price and remove from chest
				iItemValue = GetPrice(oItem, iCount)
				;debug.notification("iItemValue: "+iItemValue)
				; Increase iTotalValue to limit the total value of each "sale"
				;pppDrunkenDragonChestRefA.RemoveItem(oItem, 1)
				pppDrunkenDragonSafeRefA.AddItem(Gold, iItemValue, true)
				iTotalValue = iTotalValue + iItemValue
			endIf
		endWhile
		iTotalValue = 0
	endIf
	SetUpdate()
endEvent

; Enumerates the items in the items chest.
; Returns an index for an item in the chest, or -1 if the chest is empty.
int Function enumItems()
	int iTotalItems = pppDrunkenDragonChestRefA.GetNumItems()
	if (iTotalItems > 0)
		int iRand = Utility.RandomInt(0,iTotalItems - 1)

		; Make sure that we don't sell any gold in the chest
		form oItem = pppDrunkenDragonChestRefA.GetNthForm(iRand)
		if (oItem.getFormID() == 15) ;gold
			if (iTotalItems == iRand) ; Gold is the last item in the chest
				iRand = iRand - 1 ; should return -1 if gold is the only item in the chest
				return iRand
			else ; gold is not the last item in the chest
				iRand = iRand + 1 ; return the next item after gold
				return iRand
			endIf
		endIf
		return iRand
	else
		return -1
	endIf
endFunction

; Calculate the price for the item being sold
; Takes an item and count and returns a price
; NOTE: this removes iCount number of item from the chest
int Function GetPrice(form item, int iCount)
	; Enchanted weapons are calculated based on the following formula:
	; TotalPrice = ItemBasePrice + (ItemCharge * .12) + (EnchantmentBasePrice * 8)
	; It's off from the list price by 0-7 gold.
	; Based off information from UESP.net/Wiki/Skyrim:Generic_Magic_Weapons
	; and the associated talk page.
	ObjectReference oItem = pppDrunkenDragonChestRefA.dropObject(item,1)
	oItem.Disable()
	oItem.Delete()
	Enchantment e = None
	int iValue = oItem.GetGoldValue()
	;debug.notification("iValue: "+iValue)
	int eValue = 0
	int iCharge = 0
	if ((oItem).getBaseObject().getType() == 41) ;41=kWeapon
		e = ((oItem).getBaseObject() as Weapon).getEnchantment()
		if (e)
			iCharge = oItem.GetItemCharge() as Int
			eValue = (e.getGoldValue() * 8) + (iCharge * 0.12) as Int
			;debug.notification("Weapon eValue: "+eValue)
		endIf
	else ;26=kArmor
		e = ((oItem).getBaseObject() as Armor).getEnchantment()
		if (e)
			eValue = e.getGoldValue()
			;debug.notification("Armor eValue: "+eValue)
		endIf
	endIf
	oItem = None
	oItem.Delete()
	pppDrunkenDragonChestRefA.RemoveItem(item,iCount - 1)
	iValue = (iValue + eValue) * iCount
	;debug.notification("iValue Calc: "+iValue)
	iValue = (iValue*2)/3
	;debug.notification("iValue Done: "+iValue)
	return iValue
endFunction

; Function to hire a vendor and set things up appropriately
Function SetVendor(actor akSpeaker)
	pppInnShopkeeperRefA.ForceRefTo(akSpeaker)
	pppDrunkenDragonInnQuest.SetObjectiveCompleted(10)
	If !(akSpeaker.IsInFaction(DismissedFollowerFaction))
		(DialogueFollower as DialogueFollowerScript).DismissFollower(0, 0)
	EndIf
	SetUpdate()
endFunction

; Function to dismiss a vendor.
Function ClearVendor()
	pppInnShopkeeperRefA.Clear()
	pppDrunkenDragonInnQuest.SetObjectiveCompleted(10, False)
	pppDrunkenDragonInnQuest.SetObjectiveDisplayed(10)
	UnregisterForUpdateGameTime()
endFunction

; Shutdown to (kind of) gracefully uninstall.
;Function Uninstall()
	; Assign variables
	
	; Move actors out of Store
	
	; Reset actor factions to default
	
	; Empty store chest and safe
;	pppDrunkenDragonChestRefA.removeAllItems(PlayerRef)
;	pppDrunkenDragonSafeRefA.RemoveAllItems(PlayerRef)
;endFunction